@@ -33,6 +33,9 @@
 from binascii import a2b_hex
 from struct import pack, unpack
 from math import log
+from MidiEvents import *
+from mingus.core.diatonic import basic_keys
+from mingus.containers.Note import Note
 
 class MidiTrack():
 	"""This class is used to generate MIDI events from the
@@ -56,7 +59,7 @@ class MidiTrack():
 		"""Converts a Note object to a midi event and adds it \
 to the track_data. You can set Note.parameters["velocity"] to adjust the \
 speed with which the note should be hit [0-128]."""
-		velocity = 100
+		velocity = 64
 		if hasattr(note, "dynamics"):
 			if 'velocity' in note.dynamics:
 				velocity = note.dynamics["velocity"]
@@ -78,6 +81,8 @@ equivalent midi events and adds it to the track_data."""
 	def play_Bar(self, channel, bar):
 		"""Converts a Bar object to MIDI events and writes them \
 to the track_data."""
+		self.set_meter(bar.meter)
+		self.set_key(bar.key)
 		for x in bar:
 			tick = int(round((1.0 / x[1] * 288)))
 			if x[2] is None or len(x[2]) == 0:
@@ -101,8 +106,13 @@ them to the track_data."""
 			self.play_Bar(channel, bar)
 
 
-	def stop_Note(self, channel, note, velocity = 64):
+	def stop_Note(self, channel, note):
 		"""Adds a note_off event for note to event_track"""
+		velocity = 64
+		if hasattr(note, "dynamics"):
+			if 'velocity' in note.dynamics:
+				velocity = note.dynamics["velocity"]
+
 		self.track_data += self.note_off(channel, int(note) + 12,
 					velocity)
 
@@ -135,7 +145,7 @@ you'll have to call this function when you're done \
 adding data (when you're not using get_midi_data)."""
 		chunk_size = a2b_hex("%08x" % (len(self.track_data) +\
 				len(self.end_of_track())))
-		return "MTrk" + chunk_size
+		return TRACK_HEADER + chunk_size
 
 	def get_midi_data(self):
 		"""Returns the MIDI data in bytes for this track. \
@@ -161,15 +171,15 @@ meta event."""
 
 	def note_off(self, channel, note, velocity):
 		"""Returns bytes for a `note off` event."""
-		return self.midi_event(8, channel, note, velocity)
+		return self.midi_event(NOTE_OFF, channel, note, velocity)
 
 	def note_on(self, channel, note, velocity):
 		"""Returns bytes for a `note_on` event."""
-		return self.midi_event(9, channel, note, velocity)
+		return self.midi_event(NOTE_ON, channel, note, velocity)
 
 	def controller_event(self, channel, contr_nr, contr_val):
 		"""Returns the bytes for a MIDI controller event."""
-		return self.midi_event(11, channel, contr_nr, contr_val)
+		return self.midi_event(CONTROLLER, channel, contr_nr, contr_val)
 
 	def reset(self):
 		"""Resets track_data and delta_time."""
@@ -187,13 +197,13 @@ variable length byte."""
 	def select_bank(self, channel, bank):
 		"""Returns the MIDI event for a select bank \
 controller event."""
-		return self.controller_event(1, channel, bank)
+		return self.controller_event(BANK_SELECT, channel, bank)
 
 
 	def program_change_event(self, channel, instr):
 		"""Returns the bytes for a program change \
 controller event."""
-		return self.midi_event(12, channel, instr)
+		return self.midi_event(PROGRAM_CHANGE, channel, instr)
 
 	def set_tempo(self, bpm):
 		"""Converts the bpm to a midi event and writes it to the track_data"""
@@ -206,8 +216,36 @@ controller event."""
 		"""and returns tempo event."""
 		ms_per_min = 60000000
 		mpqn = a2b_hex("%06x" % (ms_per_min / bpm))
-		return self.delta_time + "\xff\x51\x03" + mpqn
+		return self.delta_time + META_EVENT + SET_TEMPO + \
+				"\x03" + mpqn
+	def set_meter(self, meter = (4,4)):
+		"""Adds a time signature event for meter to track_data"""
+		self.track_data += self.time_signature_event(meter)
+
+	def time_signature_event(self, meter = (4,4)):
+		"""Returns a time signature event for meter."""
+		numer = a2b_hex("%02x" % meter[0])
+		denom = a2b_hex("%02x" % int(log(meter[1], 2)))
+		return self.delta_time + META_EVENT + TIME_SIGNATURE + \
+				"\x04" + numer + denom + "\x18\x08" 
+
+	def set_key(self, key = 'C'):
+		"""Adds a key signature event to the track_data """
+		if isinstance(key, Note):
+			key = key.name
+		self.track_data += self.key_signature_event(key)
+
+	def key_signature_event(self, key = 'C'):
+		"""Returns the bytes for a key signature event."""
+		val = basic_keys.index(key) - 6
+		if val < 0:
+			val = 256 + val
+
+		key = a2b_hex("%02x" % val)
+		return self.delta_time + META_EVENT + KEY_SIGNATURE + \
+				"\x02" + key + "\x00"
 		
+
 	def int_to_varbyte(self, value):
 		"""A lot of MIDI variables can be of variable length. \
 This method converts an integer into a variable length byte. \
